import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.JOptionPane;

/*************************************************************************/
/*        Pass the output file name in the constructor                   */
/*        Pass the input directory name in the mainProcessing method     */
/*************************************************************************/

public class Analyzer {

	public Analyzer(String outFileName) {
		this.outFileName = outFileName;
		setBufferedWriter(this.outFileName);
		
		treeMap = new TreeMap<String, Integer>();
		treeSet = new TreeSet(new Comparator() {
			public int compare(Object obj1, Object obj2) {
				int compareValue = ((Comparable) ((Map.Entry)obj1).getValue()).compareTo(((Map.Entry)obj2).getValue());
				if(compareValue != 0) {
					return compareValue;
				}
				else {
					return ((Comparable) ((Map.Entry)obj1).getKey()).compareTo(((Map.Entry)obj2).getKey());
				}
			}
		});
	}
	
	public void setBufferedReader(String inFileName) {
		
		try {
			bufferedReader = new BufferedReader(new InputStreamReader(
				new FileInputStream(inFileName), "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	public void setBufferedWriter(String outFileName) {	
		try {
			bufferedWriter = new BufferedWriter(new OutputStreamWriter(
				new FileOutputStream(outFileName), "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	public void closeBufferedReader() {
		try {
			bufferedReader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void closeBufferedWriter() {
		try {
			bufferedWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void mainProcessing(String inputDir) {
		traverse(inputDir);
		createSortedTreeSet();
		closeBufferedWriter();
	}
	
	private void traverse(String inputDir) {
		
		File inDir = new File(inputDir);
		
		if (!inDir.exists()) {
			JOptionPane.showMessageDialog(null, "Doesn't Exist");
			System.exit(-1);
		}

		if (inDir.isDirectory()) {
			
			String fileList[] = inDir.list();
			for (int i = 0; i < fileList.length; i++) {
				
				String fileName = fileList[i];
				String inputFile = inputDir + File.separator + fileName;
			
				File inFile = new File(inputFile);
				if (inFile.isDirectory()) {
					traverse(inputDir);
				}
				
				else {
					setBufferedReader(inputFile);
					System.out.println(inputFile);
					processFile();
					closeBufferedReader();
				}
			}
		} 
		
		else {
			String inFileName = inDir.getAbsolutePath();
			setBufferedReader(inFileName);
			System.out.println(inFileName);
			processFile();
			closeBufferedReader();
		}
	}
	
	private void processFile() {
		
		String str = "";
		try {
			while( (str = bufferedReader.readLine()) != null ) { 
				
				/*
				 * \u00A0 == nbsp
				 * \u0964 == dari |
				 * \u09D7 == Multiplication Sign
				 * \u09F7 == Division Sign
				 * \u00A9 == copyright sign
				 * \u00D6 == Odbhut Symbol
				 * \u00C6 == Odbhut Symbol
				 */
				StringTokenizer stringTokenizer = new StringTokenizer(str, 
					" \u00A0 \u0964 \u00D7 \u00F7 \u00A9 \u00D6 \u00C6 " +
					" ? ! , . : ; @ # $ % ^ & * _ + = ~ ` -  " +
					" ( ) { } [ ] @ # $ % ( ) { } [ ]   ' / \\ \t > <   " +
					" A B C D E F G H I J K L M N O P Q R S T U V W X Y Z " +
					" a b c d e f g h i j k l m n o p q r s t u v w x y z " +
					" 1 2 3 4 5 6 7 8 9 0 " +
					" \u09E6 \u09E7 \u09E8 \u09E9 \u09EA \u09EB \u09EC \u09ED \u09EE \u09EF" );
				
				while(stringTokenizer.hasMoreElements()) {
					
					String word = stringTokenizer.nextElement().toString();
					if(treeMap.containsKey(word)) {
						int count = (int) treeMap.get(word);
						++count;
						treeMap.put(word, new Integer(count));
					}
					else {
						treeMap.put(word, new Integer(1));
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void createSortedTreeSet() {
		
		treeSet.addAll(treeMap.entrySet());
		for(Iterator iter = treeSet.descendingIterator(); iter.hasNext();) {
			Map.Entry entry = (Map.Entry) iter.next();
			try {
				String word = entry.getKey().toString();
				boolean flag = false; // flag == false when the word contains only Bangla Characters
				for(int i = 0; i < word.length(); ++i) {
					if(word.charAt(i) < 2433 || word.charAt(i) > 2527) {
						flag = true;
					}
				}
				if(flag == false) {
					bufferedWriter.write(entry.getKey() + " " + entry.getValue());
					bufferedWriter.write("\n");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void main(String[] args) {
		
		Analyzer application = new Analyzer("test/out/all.txt");
		application.mainProcessing("test/in");
	}
	
	private String outFileName;
	private BufferedReader bufferedReader;
	private BufferedWriter bufferedWriter;
	private TreeMap<String, Integer>treeMap;
	private TreeSet treeSet;
}